@@ -613,6 +613,10 @@ static void _read_attribute_params(VFormatAttribute *attr, char **p, int *format
 			osync_trace(TRACE_INTERNAL, "invalid character found in parameter spec: \"%i\" String so far: %s", lp[0], str->str);
 			g_string_assign (str, "");
 			_skip_until (&lp, ":;");
+			if (*lp == '\r') {
+				osync_trace(TRACE_INTERNAL, "string ended unexpectedly with \\r, skipping it");
+				break;
+			}
 		}
 	}
 
@@ -823,7 +827,7 @@ char *vformat_escape_string (const char *s, VFormatType type)
 			 * on vcard 2.1, unless it is in the end of a value.
 			 * See comments above for a better explanation
 			**/
-			if (*p != '\0' && type == VFORMAT_CARD_21) {
+			if (*p != '\0' && (type == VFORMAT_CARD_21 || type == VFORMAT_EVENT_10)) {
 				osync_trace(TRACE_INTERNAL, "[%s]We won't escape backslashes", __func__);
 				str = g_string_append_c(str, *p);
 			}
@@ -1016,12 +1020,13 @@ char *vformat_to_string (VFormat *evc, VFormatType type)
 			else {
 				attr_str = g_string_append_c (attr_str, ';');
 				/**
-				 * The "TYPE=" is optional skip it.
+				 * The "TYPE=" is optional skip it. (2.1.2 Property)
 				 * LOGO, PHOTO and SOUND multimedia formats MUST
 				 * have a "TYPE=" parameter
 				**/
 				gboolean must_have_type = FALSE;
-				if (!g_ascii_strcasecmp (attr->name, "PHOTO") || !g_ascii_strcasecmp (attr->name, "LOGO") || !g_ascii_strcasecmp (attr->name, "SOUND") )
+				if (!g_ascii_strcasecmp (attr->name, "PHOTO") || !g_ascii_strcasecmp (attr->name, "LOGO") || !g_ascii_strcasecmp (attr->name, "SOUND") ||
+						!g_ascii_strcasecmp (attr->name, "AALARM") || !g_ascii_strcasecmp (attr->name, "PALARM") )
 					must_have_type = TRUE;
 				if ( must_have_type || g_ascii_strcasecmp (param->name, "TYPE") )
 					attr_str = g_string_append (attr_str, param->name);
