@@ -50,13 +50,40 @@
 void handle_vcal_type_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
 {
 	osync_trace(TRACE_INTERNAL, "Handling %s parameter", vformat_attribute_param_get_name(param));
-	osync_xmlfield_set_attr(xmlfield, "Type", vformat_attribute_param_get_nth_value(param, 0));
+	const char *type = vformat_attribute_param_get_nth_value(param, 0);
+
+	// in case of type was within an alarm field, we call it FormatType
+	if (!strncmp(osync_xmlfield_get_name(xmlfield), "Alarm", 5)) {
+		if (!strcasecmp("PCM", type)) {
+			osync_xmlfield_set_attr(xmlfield, "FormatType", "audio/basic");
+		} else if (!strcasecmp("WAVE", type)) {
+			osync_xmlfield_set_attr(xmlfield, "FormatType", "audio/x-wav");
+		} else if (!strcasecmp("X-EPOCSOUND", type)) {
+			osync_trace(TRACE_INTERNAL, "skipping %s parameter", type);
+		} else {
+			osync_xmlfield_set_attr(xmlfield, "FormatType", type);
+		}
+	} else {
+		osync_xmlfield_set_attr(xmlfield, "Type", type);
+	}
 }
 
 void handle_vcal_value_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
 {
 	osync_trace(TRACE_INTERNAL, "Handling %s parameter", vformat_attribute_param_get_name(param));
-	osync_xmlfield_set_attr(xmlfield, "Value", vformat_attribute_param_get_nth_value(param, 0));
+	const char *value = vformat_attribute_param_get_nth_value(param, 0);
+
+	// in case of value was within an alarm field, we call it AttachValue
+	// TODO check if base64 and inline is working
+	if (!strncmp(osync_xmlfield_get_name(xmlfield), "Alarm", 5)) {
+		if (!strcasecmp("URL", value)) {
+			osync_xmlfield_set_attr(xmlfield, "AttachValue", "URI");
+		} else {
+			osync_xmlfield_set_attr(xmlfield, "AttachValue", value);
+		}
+	} else {
+		osync_xmlfield_set_attr(xmlfield, "Value", value);
+	}
 }
 
 void handle_vcal_encoding_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
@@ -113,30 +140,42 @@ void handle_vcal_expect_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
 
 
 /* BEGIN: vcalendar10 only attributes -> xml */
-OSyncXMLField *handle_vcal_aalarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error) 
-{ 
-	osync_trace(TRACE_INTERNAL, "Handling aalarm attribute");
-	OSyncXMLField *xmlfield = osync_xmlfield_new(xmlformat, "Alarm", error);
+OSyncXMLField *handle_vcal_alarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error) 
+{
+	OSyncXMLField *xmlfield = NULL;	
+	const char *name = vformat_attribute_get_name(attr);
+
+	if (!strcasecmp(name, "AALARM")) {
+		osync_trace(TRACE_INTERNAL, "Handling aalarm attribute");
+		xmlfield = osync_xmlfield_new(xmlformat, "AlarmAudio", error);
+		osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "AUDIO");
+	} else if (!strcasecmp(name, "DALARM")) {
+		osync_trace(TRACE_INTERNAL, "Handling dalarm attribute");
+		xmlfield = osync_xmlfield_new(xmlformat, "AlarmDisplay", error);
+		osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "DISPLAY");
+	} else if (!strcasecmp(name, "MALARM")) {
+		osync_trace(TRACE_INTERNAL, "Handling malarm attribute");
+		xmlfield = osync_xmlfield_new(xmlformat, "AlarmEmail", error);
+		osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "EMAIL");
+	} else if (!strcasecmp(name, "PALARM")) {
+		osync_trace(TRACE_INTERNAL, "Handling palarm attribute");
+		xmlfield = osync_xmlfield_new(xmlformat, "AlarmProcedure", error);
+		osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "PROCEDURE");
+	}
+
 	if(!xmlfield) {
 		osync_trace(TRACE_ERROR, "%s: %s" , __func__, osync_error_print(error));
 		return NULL;
 	} 
 
-	osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "AUDIO");
-	osync_xmlfield_set_key_value(xmlfield, "AlarmTrigger", vformat_attribute_get_nth_value(attr, 0)); 
-	return xmlfield; 
-}
+	if (!strcasecmp(name, "AALARM") || !strcasecmp(name, "PALARM")) {
+		osync_xmlfield_set_key_value(xmlfield, "AlarmAttach", vformat_attribute_get_nth_value(attr, 3));
+	} else {
+		osync_xmlfield_set_key_value(xmlfield, "AlarmDescription", vformat_attribute_get_nth_value(attr, 3));
+	}
 
-OSyncXMLField *handle_vcal_dalarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error) 
-{ 
-	osync_trace(TRACE_INTERNAL, "Handling dalarm attribute");
-	OSyncXMLField *xmlfield = osync_xmlfield_new(xmlformat, "Alarm", error);
-	if(!xmlfield) {
-		osync_trace(TRACE_ERROR, "%s: %s" , __func__, osync_error_print(error));
-		return NULL;
-	} 
-
-	osync_xmlfield_set_key_value(xmlfield, "AlarmAction", "DISPLAY");
+	osync_xmlfield_set_key_value(xmlfield, "AlarmRepeat", vformat_attribute_get_nth_value(attr, 2));
+	osync_xmlfield_set_key_value(xmlfield, "AlarmRepeatDuration", vformat_attribute_get_nth_value(attr, 1));
 	osync_xmlfield_set_key_value(xmlfield, "AlarmTrigger", vformat_attribute_get_nth_value(attr, 0)); 
 	return xmlfield; 
 }
@@ -152,6 +191,36 @@ OSyncXMLField *handle_vcal_rrule_attribute(OSyncXMLFormat *xmlformat, VFormatAtt
 
 
 /* BEGIN: xml -> vcalendar10 parameters */
+void handle_xml_vcal_attachvalue_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
+{
+	osync_trace(TRACE_INTERNAL, "Handling AttachValue xml parameter");
+	const char *content = osync_xmlfield_get_attr(xmlfield, "AttachValue");
+	if (content) {
+		if(!strcmp(content, "URI")) {
+			vformat_attribute_add_param_with_value(attr, "VALUE", "URL");
+		} else {
+			vformat_attribute_add_param_with_value(attr, "VALUE", content);
+		}
+	} else {
+		osync_trace(TRACE_INTERNAL, "Warning: No AttachValue parameter found!");
+	}
+}
+
+void handle_xml_vcal_formattype_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
+{
+	osync_trace(TRACE_INTERNAL, "Handling FormatType xml parameter");
+	const char *content = osync_xmlfield_get_attr(xmlfield, "FormatType");
+	if (content) {
+		if(!strcmp(content, "audio/x-wav")) {
+			vformat_attribute_add_param_with_value(attr, "TYPE", "WAVE");
+		} else {
+			vformat_attribute_add_param_with_value(attr, "TYPE", content);
+		}
+	} else {
+		osync_trace(TRACE_INTERNAL, "Warning: No FormatType parameter found!");
+	}
+}
+
 void handle_xml_vcal_rsvp_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
 {
 	osync_trace(TRACE_INTERNAL, "Handling Rsvp xml parameter");
@@ -164,6 +233,21 @@ void handle_xml_vcal_rsvp_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfi
 		vformat_attribute_add_param_with_value(attr, "RSVP", content);
 	}
 }
+
+void handle_xml_vcal_value_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
+{
+	osync_trace(TRACE_INTERNAL, "Handling Value xml parameter");
+	
+	// xmlformat-event stores VALUE=DATE-TIME or VALUE=DURATION for alarms
+	// but vcal10 doesn't know something like this, so we skip it here
+	const char *alarm = osync_xmlfield_get_key_value(xmlfield, "AlarmAction");
+	if (alarm)
+		return;
+
+	// normal case
+	const char *content = osync_xmlfield_get_attr(xmlfield, "Value");
+	vformat_attribute_add_param_with_value(attr, "VALUE", content);
+}
 /* END: xml -> vcalendar10 parameters */
 
 
@@ -174,6 +258,97 @@ VFormatAttribute *handle_xml_vcal_rrule_attribute(VFormat *vevent, OSyncXMLField
 	osync_trace(TRACE_INTERNAL, "Handling \"RRULE\" xml attribute");
 	return convert_xml_rrule_to_vcal(vevent, xmlfield, "RRULE", encoding); 
 }
+
+VFormatAttribute *handle_xml_vcal_alarm_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
+{
+	osync_trace(TRACE_INTERNAL, "Handling \"%s\" xml attribute", osync_xmlfield_get_name(xmlfield));
+	VFormatAttribute *attr = NULL;
+
+	osync_bool isruntime = FALSE;
+
+	const char *action = osync_xmlfield_get_key_value(xmlfield, "AlarmAction");
+	const char *trigger = osync_xmlfield_get_key_value(xmlfield, "AlarmTrigger");
+
+	// trigger and action are required
+	if (trigger && action) {
+
+		// create new attribute
+		if (!strcasecmp(action, "AUDIO")) {
+			attr = vformat_attribute_new(NULL, "AALARM");
+		} else if (!strcasecmp(action, "DISPLAY")) {
+			attr = vformat_attribute_new(NULL, "DALARM");
+		} else if (!strcasecmp(action, "EMAIL")) {
+			attr = vformat_attribute_new(NULL, "MALARM");
+		} else if (!strcasecmp(action, "PROCEDURE")) {
+			attr = vformat_attribute_new(NULL, "PALARM");
+		} else {
+			osync_trace(TRACE_INTERNAL, "Error: Could not parse action attribute of ALARM xmlfield");
+			return NULL;
+		}
+	} else {
+		osync_trace(TRACE_INTERNAL, "Error: No trigger or action property found");
+		return NULL;
+	}
+
+	// check if trigger is a valid timestamp or duration
+	if (strlen(trigger) >= 15)
+		isruntime = TRUE;	
+
+	if (isruntime == TRUE) {
+		vformat_attribute_add_value(attr, trigger);
+	} else {
+		const char *dtstart;
+		char *runtime = NULL;
+		time_t dtstarted; 
+		int duration;
+
+
+		// find DateStarted XMLField		
+		OSyncXMLField *tmpfield = NULL;
+		for (tmpfield = xmlfield; tmpfield != NULL; tmpfield = osync_xmlfield_get_next(tmpfield)) {
+			if (!strcasecmp(osync_xmlfield_get_name(tmpfield), "DateStarted")) {
+				dtstart = osync_xmlfield_get_key_value(tmpfield, "Content");
+			}
+		}
+
+		// we cannot calculate a new timestamp without a dtstart property :(
+		if (!dtstart) {
+			osync_trace(TRACE_INTERNAL, "Error: No dtstart property found, unable to create alarm field");
+			return NULL;
+		}
+		
+		duration = osync_time_alarmdu2sec(trigger);	
+		dtstarted = osync_time_vtime2unix(dtstart, 0);
+
+		dtstarted += duration;
+		runtime = osync_time_unix2vtime(&dtstarted);
+
+		if (!osync_time_isutc(dtstart)) {
+			osync_trace(TRACE_INTERNAL, "WARNNING: timestamp is not UTC - converting reminder to localtime");
+			char *utc_runtime = runtime;
+			runtime = osync_time_vtime2localtime(utc_runtime, 0);
+			g_free(utc_runtime);
+		}
+
+		vformat_attribute_add_value(attr, runtime);
+		g_free(runtime);
+	}
+
+	add_value(attr, xmlfield, "AlarmRepeatDuration", encoding);
+	add_value(attr, xmlfield, "AlarmRepeat", encoding);
+	
+	// AALARM and PALARM allows attachments
+	// DALARM and MALARM allows descriptions
+	if (!strcasecmp(action, "AUDIO") || !strcasecmp(action, "PROCEDURE")) {
+		add_value(attr, xmlfield, "AlarmAttach", encoding);
+	} else {
+		add_value(attr, xmlfield, "AlarmDescription", encoding);
+	}
+
+	vformat_add_attribute(vevent, attr);
+
+	return attr;
+}
 /* END: xml -> vcalendar10 only attributes */
 
 
@@ -289,9 +464,28 @@ void handle_sent_by_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
 
 void handle_tzid_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
 {
-	osync_trace(TRACE_INTERNAL, "Handling %s parameter %s\n", vformat_attribute_param_get_name(param));
+	osync_trace(TRACE_INTERNAL, "Handling %s parameter", vformat_attribute_param_get_name(param));
 	osync_xmlfield_set_attr(xmlfield, "TimezoneID", vformat_attribute_param_get_nth_value(param, 0));
 }
+
+void handle_value_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
+{
+	const char *value = vformat_attribute_param_get_nth_value(param, 0);
+	osync_trace(TRACE_INTERNAL, "Handling Value parameter %s\n", value);
+
+	// in case of value was within an alarm field, we call it AttachValue
+	// TODO check if base64 and inline is working
+	if (!strncmp(osync_xmlfield_get_name(xmlfield), "Alarm", 5)) {
+		if (!strcasecmp("DURATION", value) || !strcasecmp("DATE-TIME", value)) {
+			osync_xmlfield_set_attr(xmlfield, "Value", value);
+		} else {
+			osync_xmlfield_set_attr(xmlfield, "AttachValue", value);
+		}
+	} else {
+		osync_xmlfield_set_attr(xmlfield, "Value", value);
+	}
+
+}
 /* END: vcalendar20 parameters -> xml */
 
 
@@ -714,12 +908,6 @@ void handle_xml_value_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
 
 
 /* BEGIN: xml -> vcalendar20 (and vcalendar10) attributes */
-VFormatAttribute *handle_xml_alarm_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
-{
-	// TODO
-	return NULL;
-}
-
 VFormatAttribute *handle_xml_prodid_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	return handle_xml_attribute_simple_content(vevent, xmlfield, "PRODID", encoding);
@@ -929,6 +1117,9 @@ void handle_alarm_action_attribute(OSyncXMLField *xmlfield, VFormatAttribute *at
 	} else if(!strcmp(vformat_attribute_get_nth_value(attr, 0),"PROCEDURE")) {
 		osync_xmlfield_set_name(xmlfield, "AlarmProcedure");
 	}
+
+	// set AlarmAction
+	handle_simple_xmlfield(xmlfield, attr, "AlarmAction");
 }
 
 void handle_alarm_attach_attribute(OSyncXMLField *xmlfield, VFormatAttribute *attr) 
@@ -972,13 +1163,21 @@ void handle_alarm_trigger_attribute(OSyncXMLField *xmlfield, VFormatAttribute *a
 /* BEGIN: xml -> alarm attributes (vcalendar20) */
 VFormatAttribute *handle_xml_alarm_trigger_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
-	return handle_xml_attribute_simple_content(vevent, xmlfield, "TRIGGER", encoding);
+	VFormatAttribute *attr = vformat_attribute_new(NULL, "TRIGGER");
+	add_value(attr, xmlfield, "AlarmTrigger", encoding);
+	if (osync_xmlfield_get_attr(xmlfield, "Value"))
+		handle_xml_value_parameter(attr, xmlfield);
+	if (osync_xmlfield_get_attr(xmlfield, "RelatedType"))
+		handle_xml_related_parameter(attr, xmlfield);
+
+	vformat_add_attribute(vevent, attr);
+	return attr;
 }
 
 VFormatAttribute *handle_xml_alarm_repeat_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "REPEAT");
-	add_value(attr, xmlfield, NULL, encoding);
+	add_value(attr, xmlfield, "AlarmRepeat", encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -986,7 +1185,7 @@ VFormatAttribute *handle_xml_alarm_repeat_attribute(VFormat *vevent, OSyncXMLFie
 VFormatAttribute *handle_xml_alarm_duration_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "DURATION");
-	add_value(attr, xmlfield, NULL, encoding);
+	add_value(attr, xmlfield, "AlarmRepeatDuration", encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -994,8 +1193,7 @@ VFormatAttribute *handle_xml_alarm_duration_attribute(VFormat *vevent, OSyncXMLF
 VFormatAttribute *handle_xml_alarm_action_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "ACTION");
-	/* FIXME add_Value() #3 NULL is wrong */
-	add_value(attr, xmlfield, NULL, encoding);
+	add_value(attr, xmlfield, "AlarmAction", encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -1003,9 +1201,12 @@ VFormatAttribute *handle_xml_alarm_action_attribute(VFormat *vevent, OSyncXMLFie
 VFormatAttribute *handle_xml_alarm_attach_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "ATTACH");
-	/* FIXME add_Value() #3 NULL is wrong */
+	add_value(attr, xmlfield, "AlarmAttach", encoding);
+	if (osync_xmlfield_get_attr(xmlfield, "FormatType"))
+		handle_xml_format_type_parameter(attr, xmlfield);
+	if (osync_xmlfield_get_attr(xmlfield, "Encoding"))
+		handle_xml_encoding_parameter(attr, xmlfield);
 
-	add_value(attr, xmlfield, NULL, encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -1013,9 +1214,12 @@ VFormatAttribute *handle_xml_alarm_attach_attribute(VFormat *vevent, OSyncXMLFie
 VFormatAttribute *handle_xml_alarm_description_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "DESCRIPTION");
-	/* FIXME add_Value() #3 NULL is wrong */
+	add_value(attr, xmlfield, "AlarmDescription", encoding);
+	if (osync_xmlfield_get_attr(xmlfield, "AlternativeTextRep"))
+		handle_xml_altrep_parameter(attr, xmlfield);
+	if (osync_xmlfield_get_attr(xmlfield, "Language"))
+		handle_xml_language_parameter(attr, xmlfield);
 
-	add_value(attr, xmlfield, NULL, encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -1023,9 +1227,8 @@ VFormatAttribute *handle_xml_alarm_description_attribute(VFormat *vevent, OSyncX
 VFormatAttribute *handle_xml_alarm_attendee_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "ATTENDEE");
-	/* FIXME add_Value() #3 NULL is wrong */
-
-	add_value(attr, xmlfield, NULL, encoding);
+	add_value(attr, xmlfield, "AlarmAttendee", encoding);
+	// FIXME: add values
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -1033,9 +1236,12 @@ VFormatAttribute *handle_xml_alarm_attendee_attribute(VFormat *vevent, OSyncXMLF
 VFormatAttribute *handle_xml_alarm_summary_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding)
 {
 	VFormatAttribute *attr = vformat_attribute_new(NULL, "SUMMARY");
-	/* FIXME add_Value() #3 NULL is wrong */
+	add_value(attr, xmlfield, "AlarmSummary", encoding);
+	if (osync_xmlfield_get_attr(xmlfield, "AlternativeTextRep"))
+		handle_xml_altrep_parameter(attr, xmlfield);
+	if (osync_xmlfield_get_attr(xmlfield, "Language"))
+		handle_xml_language_parameter(attr, xmlfield);
 
-	add_value(attr, xmlfield, NULL, encoding);
 	vformat_add_attribute(vevent, attr);
 	return attr;
 }
@@ -1321,7 +1527,6 @@ void vcalendar_parse_attributes(OSyncXMLFormat *xmlformat, GList **attributes, O
 
 
 
-/* Review and remove old and unused code */
 /*
 static void xml_handle_unknown_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield)
 {
