@@ -38,13 +38,16 @@ void handle_vcal_status_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 void handle_vcal_rsvp_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 void handle_vcal_expect_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 
-OSyncXMLField *handle_vcal_aalarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error); 
-OSyncXMLField *handle_vcal_dalarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error); 
+OSyncXMLField *handle_vcal_alarm_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error); 
 OSyncXMLField *handle_vcal_rrule_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error);
 
+void handle_xml_vcal_attachvalue_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
+void handle_xml_vcal_formattype_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
 void handle_xml_vcal_rsvp_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
+void handle_xml_vcal_value_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
 
 VFormatAttribute *handle_xml_vcal_rrule_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
+VFormatAttribute *handle_xml_vcal_alarm_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 
 // vcalendar20 only
 void handle_range_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
@@ -66,6 +69,7 @@ void handle_reltype_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 void handle_member_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 void handle_fb_type_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 void handle_trigrel_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
+void handle_value_parameter(OSyncXMLField *xmlfield, VFormatParam *param);
 
 OSyncXMLField *handle_attach_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error);
 OSyncXMLField *handle_attendee_attribute(OSyncXMLFormat *xmlformat, VFormatAttribute *attr, OSyncError **error); 
@@ -123,10 +127,8 @@ void handle_xml_rsvp_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
 void handle_xml_sent_by_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
 void handle_xml_value_parameter(VFormatAttribute *attr, OSyncXMLField *xmlfield);
 
-VFormatAttribute *handle_xml_alarm_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 VFormatAttribute *handle_xml_prodid_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 VFormatAttribute *handle_xml_method_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
-VFormatAttribute *handle_xml_alarm_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 VFormatAttribute *handle_xml_geo_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 VFormatAttribute *handle_xml_dtstamp_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
 VFormatAttribute *handle_xml_description_attribute(VFormat *vevent, OSyncXMLField *xmlfield, const char *encoding);
