@@ -130,6 +130,12 @@ static void handle_uislot_parameter(OSyncXMLField *xmlfield, VFormatParam *param
 	osync_xmlfield_set_attr(xmlfield, "UI-Slot", vformat_attribute_param_get_nth_value(param, 0));
 }
 
+static void handle_value_parameter(OSyncXMLField *xmlfield, VFormatParam *param)
+{
+	osync_trace(TRACE_INTERNAL, "Handling Value parameter %s\n", vformat_attribute_param_get_name(param));
+	osync_xmlfield_set_attr(xmlfield, "Value", vformat_attribute_param_get_nth_value(param, 0));
+}
+
 /* TODO: drop unknown parameters? */
 //static void handle_unknown_parameter(OSyncXMLField *current, VFormatParam *param)
 //{
@@ -962,7 +968,7 @@ static OSyncHookTables *init_vcard_to_xmlformat(void)
 //	return;
 //}
 
-osync_bool conv_vcard_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vcard_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p, %i, %p, %s, %p)", __func__, input, inpsize, output, outpsize, free_input, config, error);
 	
@@ -1040,7 +1046,7 @@ osync_bool conv_vcard_to_xmlformat(char *input, unsigned int inpsize, char **out
 
 	*free_input = TRUE;
 	*output = (char *)xmlformat;
-	*outpsize = sizeof(xmlformat);
+	*outpsize = osync_xmlformat_size();
 
 	// XXX: remove this later?
 	osync_xmlformat_sort(xmlformat);
@@ -1051,11 +1057,6 @@ osync_bool conv_vcard_to_xmlformat(char *input, unsigned int inpsize, char **out
 	osync_trace(TRACE_SENSITIVE, "Output XMLFormat is:\n%s", str);
 	g_free(str);
 
-	if (osync_xmlformat_validate(xmlformat) == FALSE)
-		osync_trace(TRACE_INTERNAL, "XMLFORMAT CONTACT: Not valid!");
-	else
-		osync_trace(TRACE_INTERNAL, "XMLFORMAT CONTACT: VAILD");
-
 	vformat_free(vcard);
 	
 	osync_trace(TRACE_EXIT, "%s: TRUE", __func__);
@@ -1925,12 +1926,12 @@ static osync_bool conv_xmlformat_to_vcard(char *input, unsigned int inpsize, cha
 	return TRUE;
 }
 
-osync_bool conv_xmlformat_to_vcard30(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vcard30(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcard(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_CARD_30);
 }
 
-osync_bool conv_xmlformat_to_vcard21(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vcard21(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcard(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_CARD_21);
 }
