@@ -47,8 +47,10 @@ static OSyncHookTables *init_vcalendar_to_xmlformat(VFormatType target)
         // [vcal-1.0] param (same order as in sepc!)
 	insert_param_handler(hooks->parameters, "TYPE", handle_vcal_type_parameter);
 	insert_param_handler(hooks->parameters, "VALUE", handle_vcal_value_parameter); 
-	// We don't need to handle encoding/charset here
-	insert_param_handler(hooks->parameters, "ENCODING", HANDLE_IGNORE); // handle_vcal_encoding_parameter
+	// QUOTED-PRINTABLE is already handled by vformat.c
+	insert_param_handler(hooks->parameters, "ENCODING=QUOTED-PRINTABLE", HANDLE_IGNORE);
+	insert_param_handler(hooks->parameters, "ENCODING", handle_vcal_encoding_parameter);
+	// CHARSET is already handled by vformat.c
 	insert_param_handler(hooks->parameters, "CHARSET", HANDLE_IGNORE); // handle_vcal_charset_parameter
 	insert_param_handler(hooks->parameters, "LANGUAGE", handle_vcal_language_parameter);
 	insert_param_handler(hooks->parameters, "ROLE", handle_vcal_role_parameter); // (ATTENDEE)
@@ -91,13 +93,13 @@ static OSyncHookTables *init_vcalendar_to_xmlformat(VFormatType target)
 	// X-Word
 
         // [vcal-1.0] entprop (same order as in spec!)
-	insert_attr_handler(hooks->attributes, "AALARM", handle_vcal_aalarm_attribute);
+	insert_attr_handler(hooks->attributes, "AALARM", handle_vcal_alarm_attribute);
 	insert_attr_handler(hooks->attributes, "CATEGORIES", handle_categories_attribute);
 	insert_attr_handler(hooks->attributes, "CLASS", handle_class_attribute);
-	insert_attr_handler(hooks->attributes, "DALARM", handle_vcal_dalarm_attribute);
+	insert_attr_handler(hooks->attributes, "DALARM", handle_vcal_alarm_attribute);
 	insert_attr_handler(hooks->attributes, "EXDATE", handle_exdate_attribute);
-	insert_attr_handler(hooks->attributes, "MALARM", HANDLE_IGNORE);  // TODO
-	insert_attr_handler(hooks->attributes, "PALARM", HANDLE_IGNORE);  // TODO
+	insert_attr_handler(hooks->attributes, "MALARM", handle_vcal_alarm_attribute);
+	insert_attr_handler(hooks->attributes, "PALARM", handle_vcal_alarm_attribute);
 	insert_attr_handler(hooks->attributes, "RDATE", handle_rdate_attribute);
 	insert_attr_handler(hooks->attributes, "RESOURCES", handle_resources_attribute);
 	insert_attr_handler(hooks->attributes, "STATUS", handle_status_attribute);
@@ -139,7 +141,7 @@ static OSyncHookTables *init_vcalendar_to_xmlformat(VFormatType target)
 	insert_param_handler(hooks->parameters, "MEMBER", handle_member_parameter); // memberparam
 	insert_param_handler(hooks->parameters, "PARTSTAT", handle_partstat_parameter); // partstatparam
 	insert_param_handler(hooks->parameters, "RANGE", handle_range_parameter); // rangeparam
-	insert_param_handler(hooks->parameters, "RELATED", handle_trigrel_parameter); // trigrelparam -> TODO xsd
+	insert_param_handler(hooks->parameters, "RELATED", handle_trigrel_parameter); // trigrelparam
 	insert_param_handler(hooks->parameters, "RELTYPE", handle_reltype_parameter); // reltypeparam
 	insert_param_handler(hooks->parameters, "ROLE", handle_role_parameter); // roleparam
 	insert_param_handler(hooks->parameters, "RSVP", handle_rsvp_parameter); // rsvpparam
@@ -468,8 +470,11 @@ static OSyncHookTables *init_xmlformat_to_vcalendar(VFormatType target)
 
 
         // [vcal-1.0] param (same order as in spec!)
+	// special parameter handler for alarms
+	insert_xml_attr_handler(hooks->parameters, "FormatType", handle_xml_vcal_formattype_parameter); // (AALARM)
+	insert_xml_attr_handler(hooks->parameters, "AttachValue", handle_xml_vcal_attachvalue_parameter); // (AALRM)
 	// TODO -> TYPE 
-	insert_xml_attr_handler(hooks->parameters, "Value", handle_xml_value_parameter);
+	insert_xml_attr_handler(hooks->parameters, "Value", handle_xml_vcal_value_parameter);
 	insert_xml_attr_handler(hooks->parameters, "Encoding", handle_xml_encoding_parameter);
 	// TODO -> CHARSET
 	insert_xml_attr_handler(hooks->parameters, "Language", handle_xml_language_parameter);
@@ -512,13 +517,13 @@ static OSyncHookTables *init_xmlformat_to_vcalendar(VFormatType target)
 
 
         // [vcal-1.0] entprop (same order as in spec!)
-	// TODO -> AALARM
+	insert_xml_attr_handler(hooks->attributes, "AlarmAudio", handle_xml_vcal_alarm_attribute);
+	insert_xml_attr_handler(hooks->attributes, "AlarmDisplay", handle_xml_vcal_alarm_attribute);
+	insert_xml_attr_handler(hooks->attributes, "AlarmEmail", handle_xml_vcal_alarm_attribute);
+	insert_xml_attr_handler(hooks->attributes, "AlarmProcedure", handle_xml_vcal_alarm_attribute);
 	insert_xml_attr_handler(hooks->attributes, "Categories", handle_xml_categories_attribute);
 	insert_xml_attr_handler(hooks->attributes, "Class", handle_xml_class_attribute);
-	// TODO -> DALARM
 	insert_xml_attr_handler(hooks->attributes, "ExclusionDate", handle_xml_exdate_attribute);
-	// TODO -> MALARM
-	// TODO -> PALARM
 	insert_xml_attr_handler(hooks->attributes, "RecurrenceDate", handle_xml_rdate_attribute);
 	insert_xml_attr_handler(hooks->attributes, "Resources", handle_xml_resources_attribute);
 	insert_xml_attr_handler(hooks->attributes, "Status", handle_xml_status_attribute);
@@ -622,8 +627,7 @@ static OSyncHookTables *init_xmlformat_to_vcalendar(VFormatType target)
 	insert_xml_attr_handler(hooks->attributes, "RecurrenceDate", handle_xml_tz_rdate_attribute);
 	*/
 
-	//Alarm TODO
-	/*
+	// [RFC 2445] alarm props (audio, display, email, procedure)
 	insert_xml_attr_handler(hooks->attributes, "AlarmTrigger", handle_xml_alarm_trigger_attribute);
 	insert_xml_attr_handler(hooks->attributes, "AlarmRepeat", handle_xml_alarm_repeat_attribute);
 	insert_xml_attr_handler(hooks->attributes, "AlarmDuration", handle_xml_alarm_duration_attribute);
@@ -632,7 +636,6 @@ static OSyncHookTables *init_xmlformat_to_vcalendar(VFormatType target)
 	insert_xml_attr_handler(hooks->attributes, "AlarmDescription", handle_xml_alarm_description_attribute);
 	insert_xml_attr_handler(hooks->attributes, "AlarmAttendee", handle_xml_alarm_attendee_attribute);
 	insert_xml_attr_handler(hooks->attributes, "AlarmSummary", handle_xml_alarm_summary_attribute);
-	*/
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, hooks);
 	return (void *)hooks;
@@ -681,7 +684,7 @@ static osync_bool conv_vcalendar_to_xmlformat(char *input, unsigned int inpsize,
 
 	*free_input = TRUE;
 	*output = (char *)xmlformat;
-	*outpsize = sizeof(xmlformat);
+	*outpsize = osync_xmlformat_size();
 
 	// XXX: remove this later?
 	osync_xmlformat_sort(xmlformat);
@@ -692,12 +695,6 @@ static osync_bool conv_vcalendar_to_xmlformat(char *input, unsigned int inpsize,
 	osync_trace(TRACE_SENSITIVE, "Output XMLFormat is:\n%s", str);
 	g_free(str);
 
-	if (osync_xmlformat_validate(xmlformat) == FALSE)
-		osync_trace(TRACE_INTERNAL, "XMLFORMAT EVENT: Not valid!");
-	else
-		osync_trace(TRACE_INTERNAL, "XMLFORMAT EVENT: VAILD");
-
-
 	vformat_free(vcalendar);
 	
 	osync_trace(TRACE_EXIT, "%s: TRUE", __func__);
@@ -819,24 +816,29 @@ static osync_bool conv_xmlformat_to_vcalendar(char *input, unsigned int inpsize,
 	}
 	vformat_add_attribute(vcalendar, attr);
 
-	// TODO: Handle VALARM component
-
 	OSyncXMLField *xmlfield = osync_xmlformat_get_first_field(xmlformat);
 	for(; xmlfield != NULL; xmlfield = osync_xmlfield_get_next(xmlfield)) {
 
-		// Skip Alarm* and Timezone* xmlfields
-		if (strstr(osync_xmlfield_get_name(xmlfield), "Alarm")) {
-			osync_trace(TRACE_INTERNAL, "Skipping %s", osync_xmlfield_get_name(xmlfield));
-			continue;
+		// we need to call xml_handle_component_attribute for alarm nodes
+		if (strstr(osync_xmlfield_get_name(xmlfield), "Alarm") && target != VFORMAT_EVENT_10) {
+			VFormatAttribute *start_attr = vformat_attribute_new(NULL, "BEGIN");
+			vformat_add_attribute_with_value(vcalendar, start_attr, "VALARM"); 
+			xml_handle_component_attribute(hooks, vcalendar, xmlfield, std_encoding);
+			VFormatAttribute *end_attr = vformat_attribute_new(NULL, "END");
+			vformat_add_attribute_with_value(vcalendar, end_attr, "VALARM"); 
+
+		// skip method -> TODO
 		} else if (strstr(osync_xmlfield_get_name(xmlfield), "Method")) {
 			osync_trace(TRACE_INTERNAL, "Skipping %s", osync_xmlfield_get_name(xmlfield));
-			continue;
+
+		// skip timezone -> TODO
 		} else if (strstr(osync_xmlfield_get_name(xmlfield), "Timezone")) {
 			osync_trace(TRACE_INTERNAL, "Skipping %s", osync_xmlfield_get_name(xmlfield));
-			continue;
-		}
 
-		xml_handle_attribute(hooks, vcalendar, xmlfield, std_encoding);
+		// default
+		} else {
+			xml_handle_attribute(hooks, vcalendar, xmlfield, std_encoding);
+		}
 	}
 
 	osync_trace(TRACE_INTERNAL, "parsing xml attributes finished");
@@ -869,52 +871,52 @@ static osync_bool conv_xmlformat_to_vcalendar(char *input, unsigned int inpsize,
 	return TRUE;
 }
 
-osync_bool conv_xmlformat_to_vjournal(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vjournal(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcalendar(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_JOURNAL);
 }
 
-osync_bool conv_vjournal_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vjournal_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_vcalendar_to_xmlformat(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_JOURNAL);
 }
 
-osync_bool conv_xmlformat_to_vtodo10(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vtodo10(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcalendar(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_TODO_10);
 }
 
-osync_bool conv_xmlformat_to_vtodo20(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vtodo20(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcalendar(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_TODO_20);
 }
 
-osync_bool conv_vtodo10_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vtodo10_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_vcalendar_to_xmlformat(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_TODO_10);
 }
 
-osync_bool conv_vtodo20_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vtodo20_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_vcalendar_to_xmlformat(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_TODO_20);
 }
 
-osync_bool conv_xmlformat_to_vevent10(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vevent10(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcalendar(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_EVENT_10);
 }
 
-osync_bool conv_xmlformat_to_vevent20(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_xmlformat_to_vevent20(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_xmlformat_to_vcalendar(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_EVENT_20);
 }
 
-osync_bool conv_vevent10_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vevent10_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_vcalendar_to_xmlformat(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_EVENT_10);
 }
 
-osync_bool conv_vevent20_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, OSyncError **error)
+osync_bool conv_vevent20_to_xmlformat(char *input, unsigned int inpsize, char **output, unsigned int *outpsize, osync_bool *free_input, const char *config, void *userdata, OSyncError **error)
 {
 	return conv_vcalendar_to_xmlformat(input, inpsize, output, outpsize, free_input, config, error, VFORMAT_EVENT_20);
 }
